Package: bit64
Title: A S3 Class for Vectors of 64bit Integers
Version: 4.6.0-1
Authors@R: c(
    person("Michael", "Chirico", email = "michaelchirico4@gmail.com", role = c("aut", "cre")),
    person("Jens", "Oehlschlägel", role = "aut"),
    person("Leonardo", "Silvestri", role = "ctb"),
    person("Ofek", "Shilon", role = "ctb")
  )
Depends: R (>= 3.4.0), bit (>= 4.0.0)
Description: 
  Package 'bit64' provides serializable S3 atomic 64bit (signed) integers.
  These are useful for handling database keys and exact counting in +-2^63.
  WARNING: do not use them as replacement for 32bit integers, integer64 are not
  supported for subscripting by R-core and they have different semantics when
  combined with double, e.g. integer64 + double => integer64.
  Class integer64 can be used in vectors, matrices, arrays and data.frames.
  Methods are available for coercion from and to logicals, integers, doubles,
  characters and factors as well as many elementwise and summary functions.
  Many fast algorithmic operations such as 'match' and 'order' support inter-
  active data exploration and manipulation and optionally leverage caching.
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
URL: https://github.com/r-lib/bit64
Encoding: UTF-8
Imports: graphics, methods, stats, utils
Suggests: testthat (>= 3.0.3), withr
Config/testthat/edition: 3
Config/needs/development: testthat
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-01-16 14:04:20 UTC; michael
Author: Michael Chirico [aut, cre],
  Jens Oehlschlägel [aut],
  Leonardo Silvestri [ctb],
  Ofek Shilon [ctb]
Maintainer: Michael Chirico <michaelchirico4@gmail.com>
Repository: CRAN
Date/Publication: 2025-01-16 16:00:07 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 01:04:52 UTC; unix
