# Generated by roxygen2: do not edit by hand

S3method("!",integer64)
S3method("!=",integer64)
S3method("%%",integer64)
S3method("%/%",integer64)
S3method("%in%",default)
S3method("%in%",integer64)
S3method("&",integer64)
S3method("*",integer64)
S3method("+",integer64)
S3method("-",integer64)
S3method("/",integer64)
S3method(":",default)
S3method(":",integer64)
S3method("<",integer64)
S3method("<=",integer64)
S3method("==",integer64)
S3method(">",integer64)
S3method(">=",integer64)
S3method("[",integer64)
S3method("[<-",integer64)
S3method("[[",integer64)
S3method("[[<-",integer64)
S3method("^",integer64)
S3method("length<-",integer64)
S3method("|",integer64)
S3method(abs,integer64)
S3method(all,integer64)
S3method(all.equal,integer64)
S3method(any,integer64)
S3method(aperm,integer64)
S3method(as.bitstring,integer64)
S3method(as.character,integer64)
S3method(as.data.frame,integer64)
S3method(as.double,integer64)
S3method(as.integer,integer64)
S3method(as.integer64,"NULL")
S3method(as.integer64,bitstring)
S3method(as.integer64,character)
S3method(as.integer64,double)
S3method(as.integer64,factor)
S3method(as.integer64,integer)
S3method(as.integer64,integer64)
S3method(as.integer64,logical)
S3method(as.list,integer64)
S3method(as.logical,integer64)
S3method(c,integer64)
S3method(cbind,integer64)
S3method(ceiling,integer64)
S3method(colSums,default)
S3method(colSums,integer64)
S3method(cummax,integer64)
S3method(cummin,integer64)
S3method(cumprod,integer64)
S3method(cumsum,integer64)
S3method(diff,integer64)
S3method(duplicated,integer64)
S3method(floor,integer64)
S3method(format,integer64)
S3method(hashdup,cache_integer64)
S3method(hashfin,cache_integer64)
S3method(hashfun,integer64)
S3method(hashmap,integer64)
S3method(hashmaptab,integer64)
S3method(hashmapuni,integer64)
S3method(hashmapupo,integer64)
S3method(hashpos,cache_integer64)
S3method(hashrev,cache_integer64)
S3method(hashrin,cache_integer64)
S3method(hashtab,cache_integer64)
S3method(hashuni,cache_integer64)
S3method(hashupo,cache_integer64)
S3method(identical,integer64)
S3method(is.double,default)
S3method(is.double,integer64)
S3method(is.finite,integer64)
S3method(is.infinite,integer64)
S3method(is.na,integer64)
S3method(is.nan,integer64)
S3method(is.sorted,integer64)
S3method(is.vector,integer64)
S3method(keypos,integer64)
S3method(log,integer64)
S3method(log10,integer64)
S3method(log2,integer64)
S3method(match,default)
S3method(match,integer64)
S3method(max,integer64)
S3method(mean,integer64)
S3method(median,integer64)
S3method(mergeorder,integer64)
S3method(mergesort,integer64)
S3method(mergesortorder,integer64)
S3method(min,integer64)
S3method(na.count,integer64)
S3method(nties,integer64)
S3method(nunique,integer64)
S3method(nvalid,integer64)
S3method(order,default)
S3method(order,integer64)
S3method(orderdup,integer64)
S3method(orderfin,integer64)
S3method(orderkey,integer64)
S3method(ordernut,integer64)
S3method(orderpos,integer64)
S3method(orderqtl,integer64)
S3method(orderrnk,integer64)
S3method(ordertab,integer64)
S3method(ordertie,integer64)
S3method(orderuni,integer64)
S3method(orderupo,integer64)
S3method(prank,integer64)
S3method(print,bitstring)
S3method(print,cache)
S3method(print,integer64)
S3method(prod,integer64)
S3method(qtile,integer64)
S3method(quantile,integer64)
S3method(quickorder,integer64)
S3method(quicksort,integer64)
S3method(quicksortorder,integer64)
S3method(radixorder,integer64)
S3method(radixsort,integer64)
S3method(radixsortorder,integer64)
S3method(ramorder,integer64)
S3method(ramsort,integer64)
S3method(ramsortorder,integer64)
S3method(range,integer64)
S3method(rank,default)
S3method(rank,integer64)
S3method(rbind,integer64)
S3method(rep,integer64)
S3method(round,integer64)
S3method(rowSums,default)
S3method(rowSums,integer64)
S3method(scale,integer64)
S3method(seq,integer64)
S3method(shellorder,integer64)
S3method(shellsort,integer64)
S3method(shellsortorder,integer64)
S3method(sign,integer64)
S3method(signif,integer64)
S3method(sort,integer64)
S3method(sortfin,integer64)
S3method(sortnut,integer64)
S3method(sortorderdup,integer64)
S3method(sortorderkey,integer64)
S3method(sortorderpos,integer64)
S3method(sortorderrnk,integer64)
S3method(sortordertab,integer64)
S3method(sortordertie,integer64)
S3method(sortorderuni,integer64)
S3method(sortorderupo,integer64)
S3method(sortqtl,integer64)
S3method(sorttab,integer64)
S3method(sortuni,integer64)
S3method(sqrt,integer64)
S3method(str,integer64)
S3method(sum,integer64)
S3method(summary,integer64)
S3method(tiepos,integer64)
S3method(trunc,integer64)
S3method(unipos,integer64)
S3method(unique,integer64)
S3method(xor,integer64)
export("!.integer64")
export("!=.integer64")
export("%%.integer64")
export("%/%.integer64")
export("%in%")
export("%in%.default")
export("%in%.integer64")
export("&.integer64")
export("*.integer64")
export("+.integer64")
export("-.integer64")
export("/.integer64")
export(":")
export(":.default")
export(":.integer64")
export("<.integer64")
export("<=.integer64")
export("==.integer64")
export(">.integer64")
export(">=.integer64")
export("[.integer64")
export("[<-.integer64")
export("[[.integer64")
export("[[<-.integer64")
export("^.integer64")
export("length<-.integer64")
export("|.integer64")
export(NA_integer64_)
export(abs.integer64)
export(all.equal.integer64)
export(all.integer64)
export(any.integer64)
export(as.bitstring)
export(as.bitstring.integer64)
export(as.character.integer64)
export(as.data.frame.integer64)
export(as.double.integer64)
export(as.integer.integer64)
export(as.integer64)
export(as.integer64.NULL)
export(as.integer64.bitstring)
export(as.integer64.character)
export(as.integer64.double)
export(as.integer64.factor)
export(as.integer64.integer)
export(as.integer64.integer64)
export(as.integer64.logical)
export(as.list.integer64)
export(as.logical.integer64)
export(benchmark64)
export(binattr)
export(c.integer64)
export(cache)
export(cbind.integer64)
export(ceiling.integer64)
export(colSums)
export(cummax.integer64)
export(cummin.integer64)
export(cumprod.integer64)
export(cumsum.integer64)
export(diff.integer64)
export(duplicated.integer64)
export(floor.integer64)
export(format.integer64)
export(getcache)
export(hashcache)
export(hashdup)
export(hashdup.cache_integer64)
export(hashfin)
export(hashfin.cache_integer64)
export(hashfun)
export(hashfun.integer64)
export(hashmap)
export(hashmap.integer64)
export(hashmaptab)
export(hashmaptab.integer64)
export(hashmapuni)
export(hashmapuni.integer64)
export(hashmapupo)
export(hashmapupo.integer64)
export(hashpos)
export(hashpos.cache_integer64)
export(hashrev)
export(hashrev.cache_integer64)
export(hashrin)
export(hashrin.cache_integer64)
export(hashtab)
export(hashtab.cache_integer64)
export(hashuni)
export(hashuni.cache_integer64)
export(hashupo)
export(hashupo.cache_integer64)
export(identical.integer64)
export(integer64)
export(is.double)
export(is.double.default)
export(is.double.integer64)
export(is.finite.integer64)
export(is.infinite.integer64)
export(is.integer64)
export(is.na.integer64)
export(is.nan.integer64)
export(is.sorted.integer64)
export(is.vector.integer64)
export(jamcache)
export(keypos)
export(keypos.integer64)
export(lim.integer64)
export(log.integer64)
export(log10.integer64)
export(log2.integer64)
export(match)
export(match.default)
export(match.integer64)
export(max.integer64)
export(mean.integer64)
export(median.integer64)
export(mergeorder.integer64)
export(mergesort.integer64)
export(mergesortorder.integer64)
export(min.integer64)
export(minusclass)
export(na.count.integer64)
export(newcache)
export(nties.integer64)
export(nunique.integer64)
export(nvalid.integer64)
export(optimizer64)
export(order)
export(order.default)
export(order.integer64)
export(ordercache)
export(orderdup)
export(orderdup.integer64)
export(orderfin)
export(orderfin.integer64)
export(orderkey)
export(orderkey.integer64)
export(ordernut)
export(ordernut.integer64)
export(orderpos)
export(orderpos.integer64)
export(orderqtl)
export(orderqtl.integer64)
export(orderrnk)
export(orderrnk.integer64)
export(ordertab)
export(ordertab.integer64)
export(ordertie)
export(ordertie.integer64)
export(orderuni)
export(orderuni.integer64)
export(orderupo)
export(orderupo.integer64)
export(plusclass)
export(prank)
export(prank.integer64)
export(print.bitstring)
export(print.cache)
export(print.integer64)
export(prod.integer64)
export(qtile)
export(qtile.integer64)
export(quantile.integer64)
export(quickorder.integer64)
export(quicksort.integer64)
export(quicksortorder.integer64)
export(radixorder.integer64)
export(radixsort.integer64)
export(radixsortorder.integer64)
export(ramorder.integer64)
export(ramsort.integer64)
export(ramsortorder.integer64)
export(range.integer64)
export(rank)
export(rank.default)
export(rank.integer64)
export(rbind.integer64)
export(remcache)
export(rep.integer64)
export(round.integer64)
export(rowSums)
export(runif64)
export(scale.integer64)
export(seq.integer64)
export(setcache)
export(shellorder.integer64)
export(shellsort.integer64)
export(shellsortorder.integer64)
export(sign.integer64)
export(signif.integer64)
export(sort.integer64)
export(sortcache)
export(sortfin)
export(sortfin.integer64)
export(sortnut)
export(sortnut.integer64)
export(sortordercache)
export(sortorderdup)
export(sortorderdup.integer64)
export(sortorderkey)
export(sortorderkey.integer64)
export(sortorderpos)
export(sortorderpos.integer64)
export(sortorderrnk)
export(sortorderrnk.integer64)
export(sortordertab)
export(sortordertab.integer64)
export(sortordertie)
export(sortordertie.integer64)
export(sortorderuni)
export(sortorderuni.integer64)
export(sortorderupo)
export(sortorderupo.integer64)
export(sortqtl)
export(sortqtl.integer64)
export(sorttab)
export(sorttab.integer64)
export(sortuni)
export(sortuni.integer64)
export(sqrt.integer64)
export(str.integer64)
export(sum.integer64)
export(summary.integer64)
export(table.integer64)
export(tiepos)
export(tiepos.integer64)
export(trunc.integer64)
export(unipos)
export(unipos.integer64)
export(unique.integer64)
export(xor.integer64)
if (getRversion() >= "4.2.0") S3method(mtfrm,integer64)
importFrom(bit,clone)
importFrom(bit,is.sorted)
importFrom(bit,keyorder)
importFrom(bit,keysort)
importFrom(bit,keysortorder)
importFrom(bit,mergeorder)
importFrom(bit,mergesort)
importFrom(bit,mergesortorder)
importFrom(bit,na.count)
importFrom(bit,nties)
importFrom(bit,nunique)
importFrom(bit,nvalid)
importFrom(bit,quickorder)
importFrom(bit,quicksort)
importFrom(bit,quicksortorder)
importFrom(bit,radixorder)
importFrom(bit,radixsort)
importFrom(bit,radixsortorder)
importFrom(bit,ramorder)
importFrom(bit,ramsort)
importFrom(bit,ramsortorder)
importFrom(bit,repeat.time)
importFrom(bit,setattr)
importFrom(bit,shellorder)
importFrom(bit,shellsort)
importFrom(bit,shellsortorder)
importFrom(bit,xor)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,title)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(utils,packageDescription)
importFrom(utils,strOptions)
importFrom(utils,tail)
useDynLib(bit64, .registration = TRUE, .fixes = "C_")
